#!/usr/bin/python
# $Id: transfer_monster.py 3292 2010-03-29 11:18:04Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.7.1/tools/monster/transfer_monster.py $
import os
from optparse import OptionParser
from httplib import HTTPConnection
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from ConfigParser import ConfigParser
from sys import argv
from lxml.etree import parse, tostring, fromstring
from urllib import urlopen
from xml.sax.saxutils import escape

diag_ns = 'http://www.loc.gov/zing/srw/diagnostic/'
meta_ns = 'http://meresco.com/namespace/harvester/meta'
mods_ns = 'http://www.loc.gov/mods/v3'
srw_ns  = 'http://www.loc.gov/zing/srw/'
ucp_ns  = 'info:lc/xmlns/update-v1'
oai_ns  = 'http://www.openarchives.org/OAI/2.0/'

ns = {'ddi' :'http://www.icpsr.umich.edu/DDI',
      'ddi3': 'ddi:instance:3_1',
      'r'   : 'ddi:reusable:3_1',
      's'   : 'ddi:studyunit:3_1',
      'diag': diag_ns,
      'didl': 'urn:mpeg:mpeg21:2002:02-DIDL-NS',
      'dii' : 'urn:mpeg:mpeg21:2002:01-DII-NS',
      'meta': meta_ns,
      'mods': mods_ns,
      'srw' : srw_ns,
      'ucp' : ucp_ns,
      'oai' : oai_ns
}


def showmonsters(monsters):

    print '> showmonsters'

    for m in monsters:
        print 'monster > ' + m


def transfermonsters(monsters):

    print '> transfermonsters'

    for recordid in monsters:

        if recordid.startswith('#'):
            print recordid
            continue

        print recordid,

        source_base_url = config.get('source', 'base_url')

        url = source_base_url + \
              '?version=1.1&operation=searchRetrieve&x-recordSchema=header&x-recordSchema=meta&query=' + \
              'meta.upload.id%20exact%20%22' + recordid + '%22'

        sourceresponse = parse(urlopen(url))

        if sourceresponse.xpath("//srw:numberOfRecords/text()", ns)[0] == '0':
            print '> FAIL niet gevonden (deleted?)'
            continue
            # hier delete request genereren?

        request = preparerequest(sourceresponse)
        targetresponse = sendrequest(request)

        httpstatus = targetresponse.status
        srustatus = fromstring(targetresponse.read())

        if httpstatus == 200:
            if srustatus.xpath("//ucp:operationStatus/text()", ns)[0] == "success":
                print "> SUCCESS"
            else:
                print "> FAIL", srustatus.xpath("//diag:diagnostic/diag:details/text()", ns)[0]
        else:
            print "> FAIL HTTP problem: %s" % httpstatus



def sendrequest(request):

    #print "\nREQUEST\n" + request + "\nENDREQUEST\n"#

    target_base_url = config.get('target', 'base_url')
    target_port = config.get('target', 'port')

    connection = HTTPConnection(target_base_url, target_port)

    connection.putrequest("POST", "/update")
    connection.putheader("Content-Type", "text/xml; charset=\"utf-8\"")
    connection.putheader("Content-Length", str(len(request)))
    connection.endheaders()
    connection.send(request)

    response = connection.getresponse()

    return response


def preparerequest(response):

    #print 'RESPONSE:' + tostring(response)

    identifier = response.xpath("//meta:record/meta:id/text()", ns)[0]
    repository = response.xpath("//meta:repository/meta:id/text()", ns)[0]

    # FIXME waarom meta.upload.id niet gebruiken??
    updateid = repository + ':' + identifier

    header = escape(tostring(response.xpath('//srw:extraRecordData/recordData[@recordSchema="header"]/oai:header', ns)[0]))
    meta = escape(tostring(response.xpath('//srw:extraRecordData/recordData[@recordSchema="meta"]/meta:meta', ns)[0]))
    metadata = escape(tostring(response.xpath('//srw:recordData/oai:metadata', ns)[0]))

    # spatie/newline voor <document forceert fout, dus niet doen!
    document = """<document xmlns="http://meresco.com/namespace/harvester/document">
    <part name="meta">%s</part>
    <part name="header">%s</part>
    <part name="metadata">%s</part>
    </document>""" % (meta, header, metadata)

    request = """<?xml version="1.0" encoding="UTF-8"?>
    <srw:updateRequest xmlns:srw="http://www.loc.gov/zing/srw/" xmlns:ucp="http://www.loc.gov/ucp">
    <srw:version>1.0</srw:version>
    <ucp:action>info:srw/action/1/replace</ucp:action>
    <ucp:recordIdentifier>%s</ucp:recordIdentifier>
    <srw:record>
    <srw:recordPacking>xml</srw:recordPacking>
    <srw:recordSchema>mods</srw:recordSchema>
    <srw:recordData>%s</srw:recordData>
    </srw:record>
    </srw:updateRequest>""" % (updateid, document)

    #print 'REQUEST' + request

    return request

def showconfig(configfile, config):
    print 'inhoud van', configfile, ':'
    for section in config.sections():
        print section
        for option in config.options(section):
            print '>>', option, '=', config.get(section, option)

    source_host = config.get('source', 'host')
    source_port = config.get('source', 'port')

    print '>', source_host
    print '>', source_port

    database = config.get('database', 'path')
    print '>', database
    print 'blabla'


def loadmonsters(file):

    monsters = open(file, "r")
    recordids = monsters.readlines()
    monsters.close();

    monster_list = []
    for recordid in recordids:
        monster_list.append(recordid.strip())

    return monster_list

def main():

    args = argv[1:]

    if len(args) < 2:
        print 'usage: %s <config-file> <input-file>' % os.path.basename(argv[0])
        exit()

    configfile = args[0]
    monsterfile = args[1]

    global config
    config = ConfigParser()
    config.read(configfile)
    #showconfig(configfile, config)

    monsters = loadmonsters(monsterfile)

    #showmonsters(monsters)
    transfermonsters(monsters)

if __name__ == '__main__':
    main()

